@@ -1,6 +1,4 @@
-
-use crate::source::Fps;
-use crate::source::Source;
+use crate::source::{Fps, Source};
 use crate::BinResult;
 use gifski::Collector;
 use std::path::PathBuf;
@@ -12,7 +10,10 @@ pub struct Lodecoder {
 
 impl Lodecoder {
     pub fn new(frames: Vec<PathBuf>, params: Fps) -> Self {
-        Self { frames, fps: f64::from(params.fps) * f64::from(params.speed) }
+        Self {
+            frames,
+            fps: f64::from(params.fps) * f64::from(params.speed),
+        }
     }
 }
 
