@@ -1,4 +1,4 @@
-use crate::CatResult;
+use crate::GifResult;
 use std::fmt;
 use std::io;
 use std::os::raw::c_int;
@@ -41,7 +41,7 @@ impl From<GifskiError> for io::Error {
             WRITE_ZERO => EK::WriteZero,
             INTERRUPTED => EK::Interrupted,
             UNEXPECTED_EOF => EK::UnexpectedEof,
-            _ => return io::Error::new(EK::Other, g),
+            _ => return Self::other(g),
         }.into()
     }
 }
@@ -71,9 +71,9 @@ impl From<c_int> for GifskiError {
     }
 }
 
-impl From<CatResult<()>> for GifskiError {
+impl From<GifResult<()>> for GifskiError {
     #[cold]
-    fn from(res: CatResult<()>) -> Self {
+    fn from(res: GifResult<()>) -> Self {
         use crate::error::Error::*;
         match res {
             Ok(()) => GifskiError::OK,
